@@ -96,15 +96,18 @@ class OneIwyuTest(unittest.TestCase):
       'clmode.cc': ['--driver-mode=cl', '/GF', '/Os', '/W2'],
       'conversion_ctor.cc': ['-std=c++11'],
       'deleted_implicit.cc' : ['-std=c++11'],
+      'funcptrs.cc': ['-Wno-unused'],
       'lambda_fwd_decl.cc': ['-std=c++11'],
       'lateparsed_template.cc': ['-fdelayed-template-parsing'],
       'macro_defined_by_includer.cc': [
-          '-std=c++11', '-DCOMMAND_LINE_TYPE=double'],
+          '-std=c++11', '-DCOMMAND_LINE_TYPE=double',
+          self.Include('macro_defined_by_includer-prefix.h')],
       'ms_inline_asm.cc': ['-fms-extensions'],
       'prefix_header_attribution.cc': [self.Include('prefix_header_attribution-d1.h')],
       'prefix_header_includes_add.cc': prefix_headers,
       'prefix_header_includes_keep.cc': prefix_headers,
       'prefix_header_includes_remove.cc': prefix_headers,
+      'typedef_in_template.cc': ['-std=c++11'],
     }
     include_map = {
       'alias_template.cc': ['.'],
@@ -114,6 +117,7 @@ class OneIwyuTest(unittest.TestCase):
       'backwards_includes.cc': ['.'],
       'badinc.cc': ['.'],
       'badinc-extradef.cc': ['.'],
+      'funcptrs.cc': ['.'],
       'casts.cc': ['.'],
       'catch.cc': ['.'],
       'check_also.cc': ['.'],
@@ -154,6 +158,7 @@ class OneIwyuTest(unittest.TestCase):
       'overloaded_class.cc': ['.'],
       'pch_in_code.cc': ['.'],
       'pointer_arith.cc': ['.'],
+      'pragma_associated.cc': ['.'],
       'precomputed_tpl_args.cc': ['.'],
       'prefix_header_attribution.cc': ['.'],
       'prefix_header_includes_add.cc': ['.'],
@@ -171,6 +176,7 @@ class OneIwyuTest(unittest.TestCase):
       'template_specialization.cc': ['.'],
       'typedef_chain_in_template.cc': ['.'],
       'typedef_chain_no_follow.cc': ['.'],
+      'typedef_in_template.cc': ['.'],
       'typedefs_and_resugaring.cc': ['.'],
       'unused_class_template_ctor.cc': ['.'],
       'uses_printf.cc': ['.'],
